from django.urls import path

from . import views

urlpatterns = [
    path('', views.index, name='index'),
    path('create/', views.CreateMovieView.as_view(), name='create'),
    path('update/', views.UpdateMovieView.as_view(), name='update'),
]



from extra_views import ModelFormSetView


class HotView(ModelFormSetView):

    action = 'create'
    button_text = 'Create'
    date_format = 'YYYY-MM-DD'
    errors_template = 'hot/errors.html'
    hot_template = 'hot/hot.html'
    hot_settings = {}

    # Add one minSpareRow for create action
    def get(self, request, *args, **kwargs):
        if self.action == 'create':
            self.hot_settings['minSpareRows'] = 1
        return super(HotView, self).get(request, *args, **kwargs)

    # Set number of extra forms in formset
    def get_factory_kwargs(self, **kwargs):
        factory_kwargs = super(HotView, self).get_factory_kwargs(**kwargs)
        if self.action == 'create':
            factory_kwargs['extra'] = 1
        else:
            factory_kwargs['extra'] = 0
        return factory_kwargs

    # Define queryset
    def get_queryset(self):
        if self.action == 'create':
            return self.model.objects.none()
        else:
            return super(HotView, self).get_queryset()

    # Add to the context
    def get_context_data(self, **kwargs):
        context = super(HotView, self).get_context_data(**kwargs)

        context['action'] = self.action
        context['button_text'] = self.button_text
        context['date_format'] = self.date_format
        context['hot_template'] = self.hot_template
        context['errors_template'] = self.errors_template
        context['hot_settings'] = self.hot_settings

        return context

    # Debug methods
    def formset_valid(self, formset):
        print("Form valid")
        return super(HotView, self).formset_valid(formset)

    def formset_invalid(self, formset):
        print("Form innnnnvalid")
        return super(HotView, self).formset_invalid(formset)



from django.forms import CheckboxSelectMultiple, CheckboxInput, DateInput
from django.http import HttpResponseRedirect
from django.urls import reverse, reverse_lazy

from funky_sheets.formsets import HotView

from .models import Movie


def index(request):
    return HttpResponseRedirect(reverse('update'))


class CreateMovieView(HotView):
    model = Movie
    template_name = 'examples/create.html'
    prefix = 'table'
    success_url = reverse_lazy('update')
    fields = (
        'id',
        'title',
        'director',
        'release_date',
        'parents_guide',
        'imdb_rating',
        'genre',
        'imdb_link',
    )

    factory_kwargs = {
        'widgets': {
            'release_date': DateInput(attrs={'type': 'date'}),
            'genre': CheckboxSelectMultiple(),
            'parents_guide': CheckboxInput(),
        }
    }

    hot_settings = {
        # 'columnSorting': 'true',
        'contextMenu': 'true',
        'autoWrapRow': 'true',
        'rowHeaders': 'true',
        'contextMenu': 'true',
        'search': 'true'
    }


class UpdateMovieView(CreateMovieView):
    template_name = 'examples/update.html'
    action = 'update'
    button_text = 'Update'

    hot_settings = {
        # 'columnSorting': 'true',
        'contextMenu': 'true',
        'autoWrapRow': 'true',
        'rowHeaders': 'true',
        'contextMenu': 'true',
        'search': 'true'
    }



# Generated by Django 2.1.7 on 2019-02-17 12:59

from django.db import migrations, models
import django.db.models.deletion

from examples.models import Movie, Genre, Director


def add_genres(apps, schema_editor):
    Genre.objects.create(name='Action')
    Genre.objects.create(name='Drama')
    Genre.objects.create(name='Horror')
    Genre.objects.create(name='Sci-Fi')
    Genre.objects.create(name='Thriller')


def add_directors(apps, schema_editor):
    Director.objects.create(name='Christopher Nolan')
    Director.objects.create(name='Paul W.S. Anderson')


def add_movies(apps, schema_editor):
    movie_one = Movie.objects.create(
        title='Inception',
        imdb_link='https://www.imdb.com/title/tt1375666/',
        parents_guide=True,
        release_date='2010-07-16',
        director_id=1,
        imdb_rating=8.8
    )
    movie_one.genre.add(1, 4)
    movie_two = Movie.objects.create(
        title='Interstellar',
        imdb_link='https://www.imdb.com/title/tt0816692/',
        parents_guide=True,
        release_date='2014-11-06',
        director_id=1,
        imdb_rating=8.6
    )
    movie_two.genre.add(2, 4)
    movie_three = Movie.objects.create(
        title='Event horizon',
        imdb_link='https://www.imdb.com/title/tt0119081/',
        parents_guide=True,
        release_date='1997-12-04',
        director_id=2,
        imdb_rating=6.7)
    movie_three.genre.add(3, 4, 5)


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Director',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=64, null=True)),
            ],
        ),
        migrations.CreateModel(
            name='Genre',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=16, null=True)),
            ],
        ),
        migrations.CreateModel(
            name='Movie',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(max_length=64, null=True)),
                ('imdb_link', models.URLField(null=True)),
                ('imdb_rating', models.DecimalField(decimal_places=1, max_digits=2, null=True)),
                ('parents_guide', models.BooleanField(default=False, null=True)),
                ('release_date', models.DateField(null=True)),
                ('director', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to='examples.Director')),
                ('genre', models.ManyToManyField(to='examples.Genre')),
            ],
        ),
        migrations.RunPython(add_genres, lambda apps, schema_editor: None),
        migrations.RunPython(add_directors, lambda apps, schema_editor: None),
        migrations.RunPython(add_movies, lambda apps, schema_editor: None),
    ]



from django import template
register = template.Library()


@register.filter
def widget_type(field):
    return field.field.widget.__class__.__name__


@register.filter
def int_list(field):
    return list(map(int, field))



import os
from setuptools import find_packages, setup

with open(os.path.join(os.path.dirname(__file__), 'README.rst')) as readme:
    README = readme.read()

with open(os.path.join(os.path.dirname(__file__), 'requirements.txt')) as requirements:
    REQUIREMENTS = requirements.readlines()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='django-funky-sheets',
    version='0.1.0',
    packages=find_packages(),
    include_package_data=True,
    license='MIT License',
    description='Django implementation of Handsontable spreadsheets for CRUD actions.',
    long_description=README,
    url='https://github.com/trco/django-funky-sheets',
    author='Uros Trstenjak',
    author_email='uros.trstenjak@gmail.com',
    install_requires=REQUIREMENTS,
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ],
)



from django.db import models


class Genre(models.Model):
    name = models.CharField(max_length=16, null=True)

    def __str__(self):
        return self.name


class Director(models.Model):
    name = models.CharField(max_length=64, null=True)

    def __str__(self):
        return self.name


class Movie(models.Model):
    # String fields
    title = models.CharField(max_length=64, null=True)
    imdb_link = models.URLField(null=True)
    # Number field
    imdb_rating = models.DecimalField(max_digits=2, decimal_places=1, null=True)
    # Boolean field
    parents_guide = models.BooleanField(default=False, null=True)
    # Date field
    release_date = models.DateField(null=True)
    # Relationship fields
    director = models.ForeignKey(Director, null=True, on_delete=models.CASCADE)
    genre = models.ManyToManyField(Genre)

    def __str__(self):
        return self.title
